/*
    Tucnak - VHF contest log
    Copyright (C) 2023  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/


#include "header.h"

#include "cwdaemon.h"
#include "davac5.h"
#include "fifo.h"
#include "language2.h"
#include "rc.h"


#ifdef HAVE_HIDAPI
int davac5_init(struct cwdaemon* cwda) {
    cwda->hid = NULL;
    MUTEX_LOCK(cwda->hid);
    davac5_open(cwda, 1);
    MUTEX_UNLOCK(cwda->hid);
    return 0;
}

int davac5_free(struct cwdaemon* cwda) {
    dbg("davac5_free(%p)\n", cwda);

    if (!cwda) return 0;

    if (cwda->hid == NULL) return 0;

    hid_close(cwda->hid);
    cwda->hid = NULL;
    return 0;
}


int davac5_open(struct cwdaemon* cwda, int verbose) {
    int ret;

    if (cwda->hid != NULL) {
        ret = 0;
        goto x;
    }

    if (hid_init()) {
        if (verbose) log_addf(VTEXT(T_CANT_INIT_HIDAPI));
        ret = 1;
        goto x;
    }

    int vid = 0x0d8c;
    int pid = 0x000c;

    cwda->hid = hid_open(vid, pid, NULL);
    if (!cwda->hid) {
        if (verbose) {
            log_addf(VTEXT(T_CANT_OPEN_DAVAC5), vid, pid);
        }
        davac5_free(cwda);
        dbg("davac5_open error\n");
        ret = -1;
        goto x;
    }
    else {
        ret = 0;
    }
    
    cwda->hid_state = 0x00;


x:;
    dbg("davac5_open=%d\n", ret);
    return ret;
}


int davac5_reset(struct cwdaemon* cwda) {
    davac5_ptt(cwda, 0);
    davac5_cw(cwda, 0);
    davac5_ssbway(cwda, 0);
    return 0;
}

static int davac5_write(struct cwdaemon* cwda) {
    unsigned char raw[5];
    raw[0] = 0; // Report ID
    raw[1] = 0; // HID output report
    raw[2] = cwda->hid_state;
    raw[3] = 0x0d; // data direction register (GPIO2 = input, 1,3,4 = output)
    raw[4] = 0;

    int ret = hid_write(cwda->hid, raw, 5);
    if (ret != 5) {
        const wchar_t* errw = hid_error(cwda->hid);
        log_addf("Can't write to davac5: %ls", errw);
        //VTEXT(T_CANT_WRITE_TO_DAVAC5)
        davac5_free(cwda);
    }
    return ret;
}

int davac5_cw(struct cwdaemon* cwda, int onoff) {
    int ret;

    MUTEX_LOCK(cwda->hid);
    if (!cwda || cwda->hid == NULL) {// function called from worker thread, exit silently
        ret = 1;
        goto x;
    }

    if (onoff)
        cwda->hid_state |= 0x08;
    else
        cwda->hid_state &= ~0x08;

    ret = davac5_write(cwda);
x:;
    MUTEX_UNLOCK(cwda->hid);
    return ret;
}

int davac5_ptt(struct cwdaemon* cwda, int onoff) {
    int ret;

    MUTEX_LOCK(cwda->hid);
    dbg("davac5_ptt cwda->hid=%p\n", cwda->hid);

    if (!cwda || cwda->hid == NULL) {
        ret = davac5_open(cwda, 0);
        if (ret) goto x;
    }
    if (onoff)
        cwda->hid_state |= 0x04;
    else
        cwda->hid_state &= ~0x04;

    ret = davac5_write(cwda);

x:;
    MUTEX_UNLOCK(cwda->hid);
    return ret;
}

/* 0=microphone, 1=soundcard */
int davac5_ssbway(struct cwdaemon* cwda, int onoff) {
    int ret;

    MUTEX_LOCK(cwda->hid);
    if (!cwda || cwda->hid == NULL) {
        ret = davac5_open(cwda, 0);
        if (ret) goto x;
    }
    if (onoff)
        cwda->hid_state |= 0x01;   /* soundcard only*/
    else
        cwda->hid_state &= ~0x01;  /* microphone + soundcard (no sound) */

    ret = davac5_write(cwda);

x:;
    MUTEX_UNLOCK(cwda->hid);
    return ret;
}




#endif
